#
# Fedora spec file for drush
#
# Copyright (c) 2015-2016 Shawn Iwinski <shawn@iwin.ski>
#
# License: MIT
# http://opensource.org/licenses/MIT
#
# Please preserve changelog entries
#

%global github_owner     drush-ops
%global github_name      drush
%global github_version   8.1.3
%global github_commit    fec57947d9cc3ae8978bd747cb3d60ec9864e912

%global composer_vendor  drush
%global composer_project drush

# "php": ">=5.4.5"
%global php_min_ver  5.4.5
# "consolidation/annotated-command": "~1.0.0-beta6"
%global consolidation_annotated_command_min_ver 1.0
%global consolidation_annotated_command_max_ver 2.0
# "consolidation/output-formatters": "~1.0.0-beta5"
%global consolidation_output_formatters_min_ver 1.0.0
%global consolidation_output_formatters_max_ver 1.1.0
# "pear/console_table": "~1.3.0"
%global pear_console_table_min_ver 1.3.0
%global pear_console_table_max_ver 2.0
# "psr/log": "~1.0"
#     NOTE: Min version not 1.0 because autoloader required
%global psr_log_min_ver 1.0.0-8
%global psr_log_max_ver 2.0
# "psy/psysh": "~0.6"
%global psysh_min_ver 0.6
%global psysh_max_ver 1.0
# "symfony/console": "~2.7"
# "symfony/finder": "~2.7"
# "symfony/process": "2.7.*"
# "symfony/var-dumper": "~2.7"
# "symfony/yaml": "~2.3"
#     NOTE: Min version not 2.7.0 because autoloader required
%global symfony_min_ver 2.7.1
%global symfony_max_ver 3.0

%global drush_dir    %{_datadir}/drush
%global pear_channel pear.drush.org
%global pear_name    drush

%global git_min_ver  1.7

# Build using "--with tests" to enable tests
# TODO: Figure out test issues and enable by default
%global with_tests 0%{?_with_tests:1}

%{!?phpdir:  %global phpdir  %{_datadir}/php}

Name:          %{composer_project}
Version:       %{github_version}
Release:       3%{?github_release}%{?dist}
Summary:       Command line shell and scripting interface for Drupal

Group:         Development/Libraries
License:       GPLv2+
URL:           http://www.drush.org/
Source0:       https://github.com/%{github_owner}/%{github_name}/archive/%{github_commit}/%{name}-%{github_version}-%{github_commit}.tar.gz

# error importing function definition for `BASH_FUNC_scl'
# https://github.com/drush-ops/drush/issues/2065
Patch0:        %{name}-issue2065.patch

BuildArch:     noarch
# %%{pear_phpdir} macro
BuildRequires: php-pear
%if %{with_tests}
BuildRequires: git >= %{git_min_ver}
BuildRequires: patch
# composer.json
BuildRequires: php(language)                                 >= %{php_min_ver}
BuildRequires: php-composer(consolidation/annotated-command) >= %{consolidation_annotated_command_min_ver}
BuildRequires: php-composer(consolidation/output-formatters) >= %{consolidation_output_formatters_min_ver}
BuildRequires: php-composer(pear/console_table)              >= %{pear_console_table_min_ver}
BuildRequires: php-composer(phpunit/phpunit)
#BuildRequires: php-composer(psr/log)                         >= %%{psr_log_min_ver}
BuildRequires: php-PsrLog                                    >= %{psr_log_min_ver}
BuildRequires: php-composer(psy/psysh)                       >= %{psysh_min_ver}
BuildRequires: php-composer(symfony/console)                 <  %{symfony_max_ver}
BuildRequires: php-composer(symfony/console)                 >= %{symfony_min_ver}
BuildRequires: php-composer(symfony/finder)                  <  %{symfony_max_ver}
BuildRequires: php-composer(symfony/finder)                  >= %{symfony_min_ver}
BuildRequires: php-composer(symfony/process)                 <  %{symfony_max_ver}
BuildRequires: php-composer(symfony/process)                 >= %{symfony_min_ver}
BuildRequires: php-composer(symfony/var-dumper)              <  %{symfony_max_ver}
BuildRequires: php-composer(symfony/var-dumper)              >= %{symfony_min_ver}
BuildRequires: php-composer(symfony/yaml)                    <  %{symfony_max_ver}
BuildRequires: php-composer(symfony/yaml)                    >= %{symfony_min_ver}
# phpcompatinfo (computed from version 8.1.3)
BuildRequires: php-ctype
BuildRequires: php-date
BuildRequires: php-fileinfo
BuildRequires: php-filter
BuildRequires: php-hash
BuildRequires: php-iconv
BuildRequires: php-json
BuildRequires: php-pcntl
BuildRequires: php-pcre
BuildRequires: php-posix
BuildRequires: php-reflection
BuildRequires: php-simplexml
BuildRequires: php-spl
## Autoloader
BuildRequires: php-composer(symfony/class-loader)
%endif

Requires:      git >= %{git_min_ver}
Requires:      patch
Requires:      php-cli
# composer.json
Requires:      php(language)                                 >= %{php_min_ver}
Requires:      php-composer(consolidation/annotated-command) <  %{consolidation_annotated_command_max_ver}
Requires:      php-composer(consolidation/annotated-command) >= %{consolidation_annotated_command_min_ver}
Requires:      php-composer(consolidation/output-formatters) <  %{consolidation_output_formatters_max_ver}
Requires:      php-composer(consolidation/output-formatters) >= %{consolidation_output_formatters_min_ver}
Requires:      php-composer(pear/console_table)              >= %{pear_console_table_min_ver}
Requires:      php-composer(phpunit/phpunit)
Requires:      php-composer(psr/log)                         < %{psr_log_max_ver}
#Requires:      php-composer(psr/log)                         >= %%{psr_log_min_ver}
Requires:      php-PsrLog                                    >= %{psr_log_min_ver}
Requires:      php-composer(psy/psysh)                       <  %{psysh_max_ver}
Requires:      php-composer(psy/psysh)                       >= %{psysh_min_ver}
Requires:      php-composer(symfony/console)                 <  %{symfony_max_ver}
Requires:      php-composer(symfony/console)                 >= %{symfony_min_ver}
Requires:      php-composer(symfony/finder)                  <  %{symfony_max_ver}
Requires:      php-composer(symfony/finder)                  >= %{symfony_min_ver}
Requires:      php-composer(symfony/var-dumper)              <  %{symfony_max_ver}
Requires:      php-composer(symfony/var-dumper)              >= %{symfony_min_ver}
Requires:      php-composer(symfony/yaml)                    <  %{symfony_max_ver}
Requires:      php-composer(symfony/yaml)                    >= %{symfony_min_ver}
# phpcompatinfo (computed from version 8.1.3)
Requires:      php-ctype
Requires:      php-date
Requires:      php-fileinfo
Requires:      php-filter
Requires:      php-hash
Requires:      php-iconv
Requires:      php-json
Requires:      php-pcre
Requires:      php-posix
Requires:      php-reflection
Requires:      php-simplexml
Requires:      php-spl
# Autoloader
Requires:      php-composer(symfony/class-loader)

# Weak dependencies
%if 0%{?fedora} >= 21
## composer.json: suggest
#Suggests:      php-composer(drush/config-extra)
Suggests:      php-pcntl
## phpcompatinfo
Suggests:      php-pecl(apcu)
Suggests:      php-pecl(Xdebug)
Suggests:      php-pecl(xhprof)
%endif

Provides:      php-composer(%{composer_vendor}/%{composer_project}) = %{version}

Obsoletes:     php-drush-drush < %{version}-%{release}
Provides:      php-drush-drush = %{version}-%{release}
Obsoletes:     drupal6-drush < %{version}-%{release}
Provides:      drupal6-drush = %{version}-%{release}
Obsoletes:     drupal7-drush < %{version}-%{release}
Provides:      drupal7-drush = %{version}-%{release}
Provides:      drupal8-drush = %{version}-%{release}
Provides:      php-pear(%{pear_channel}/%{pear_name}) = %{version}

# This pkg was the only one in this channel so the channel is no longer needed
Obsoletes: php-channel-drush

%description
Drush is a command line shell and Unix scripting interface for Drupal. If you
are unfamiliar with shell scripting, reviewing the documentation for your shell
(e.g. man bash) or reading an online tutorial (e.g. search for "bash tutorial")
will help you get the most out of Drush.

Drush core ships with lots of useful commands for interacting with code like
modules/themes/profiles. Similarly, it runs update.php, executes sql queries
and DB migrations, and misc utilities like run cron or clear cache.


%prep
%setup -qn %{github_name}-%{github_commit}

: Fix "error importing function definition for 'BASH_FUNC_scl'"
: https://github.com/drush-ops/drush/issues/2065
%patch0 -p1

: Remove unneeded stuff
rm -rf drush.bat misc/windrush_build

: W: wrong-file-end-of-line-encoding /usr/share/doc/drush/examples/sandwich.txt
sed -i 's/\r$//' examples/sandwich.txt


%build
: Create autoloader
cat <<'AUTOLOAD' | tee autoload.php
<?php
/**
 * Autoloader for %{name} and its' dependencies
 * (created by %{name}-%{version}-%{release}).
 *
 * @return \Symfony\Component\ClassLoader\ClassLoader
 */

if (!isset($fedoraClassLoader) || !($fedoraClassLoader instanceof \Symfony\Component\ClassLoader\ClassLoader)) {
    if (!class_exists('Symfony\\Component\\ClassLoader\\ClassLoader', false)) {
        require_once '%{phpdir}/Symfony/Component/ClassLoader/ClassLoader.php';
    }

    $fedoraClassLoader = new \Symfony\Component\ClassLoader\ClassLoader();
    $fedoraClassLoader->register();
}

$fedoraClassLoader->addPrefix('Drush\\', __DIR__.'/lib');

// Required dependencies
require_once '%{phpdir}/Consolidation/AnnotatedCommand/autoload.php';
require_once '%{phpdir}/Consolidation/OutputFormatters/autoload.php';
require_once '%{phpdir}/Psr/Log/autoload.php';
require_once '%{phpdir}/Psy/autoload.php';
require_once '%{phpdir}/Symfony/Component/Console/autoload.php';
require_once '%{phpdir}/Symfony/Component/Finder/autoload.php';
require_once '%{phpdir}/Symfony/Component/VarDumper/autoload.php';
require_once '%{phpdir}/Symfony/Component/Yaml/autoload.php';
$fedoraClassLoader->addPrefix('Console_', '%{pear_phpdir}');

return $fedoraClassLoader;
AUTOLOAD

: Mock Composer autoloader
mkdir vendor
ln -s ../autoload.php vendor/autoload.php


%install
mkdir -p %{buildroot}%{drush_dir}
cp -pr * %{buildroot}%{drush_dir}/

: Bin
mkdir -p %{buildroot}%{_bindir}
ln -s %{drush_dir}/drush %{buildroot}%{_bindir}/drush

: Completion
mkdir -p %{buildroot}%{_sysconfdir}/bash_completion.d
install -pm 0644 drush.complete.sh %{buildroot}%{_sysconfdir}/bash_completion.d/


%check
%if %{with_tests}
: Create tests bootstrap
cat <<'AUTOLOAD' | tee tests-autoload.php
<?php
$fedoraClassLoader = require_once '%{buildroot}%{drush_dir}/autoload.php';
$fedoraClassLoader->addPrefix('Unish\\', __DIR__.'/tests');
require_once '%{phpdir}/Symfony/Component/Process/autoload.php';
AUTOLOAD

: Update bootstrap.inc to use generated bootstrap
sed 's#vendor/autoload\.php#tests-autoload.php#' -i tests/bootstrap.inc

: Skip specific tests
pushd tests
    #rm -f \
    #    pmDownloadTest.php \
    #    siteSshTest.php \
    #    sql*Test.php
popd

UNISH_DRUSH=%{buildroot}%{drush_dir}/drush \
    %{_bindir}/phpunit --verbose --configuration tests
%else
: Tests skipped
: Build using "--with tests" to enable tests
%endif


%files
%{!?_licensedir:%global license %%doc}
# %%license
# See https://github.com/drush-ops/drush/pull/319
%doc *.md
%doc composer.json
%doc docs
%doc examples
%doc mkdocs.yml
%{_bindir}/drush
%{drush_dir}
%exclude %{drush_dir}/*.md
%exclude %{drush_dir}/box.json.dist
%exclude %{drush_dir}/composer.json
%exclude %{drush_dir}/docs
%exclude %{drush_dir}/drush.complete.sh
%exclude %{drush_dir}/examples
%exclude %{drush_dir}/mkdocs.yml
%exclude %{drush_dir}/tests
%exclude %{drush_dir}/unish.sh
%dir %{_sysconfdir}/bash_completion.d
     %{_sysconfdir}/bash_completion.d/drush.complete.sh


%changelog
* Fri Aug 19 2016 Shawn Iwinski <shawn@iwin.ski> - 8.1.3-3
- Add patch to fix "error importing function definition for `BASH_FUNC_scl'"

* Wed Aug 10 2016 Shawn Iwinski <shawn@iwin.ski> - 8.1.3-2
- Increase "consolidation/annotated-command" max version

* Tue Aug 09 2016 Shawn Iwinski <shawn@iwin.ski> - 8.1.3-1
- Update to 8.1.3 (RHBZ #1357097)

* Mon Jun 27 2016 Shawn Iwinski <shawn@iwin.ski> - 6.7.0-3
- EPEL6 requires "php-pear-Console-Table" instead of "php-composer(pear/console_table)"

* Sat Jun 18 2016 Shawn Iwinski <shawn@iwin.ski> - 6.7.0-2
- Add missing "php-composer(pear/console_table)" dependency (RHBZ #1347826)

* Fri May 06 2016 Shawn Iwinski <shawn@iwin.ski> - 6.7.0-1
- Update to 6.7.0

* Sun Jul 19 2015 Shawn Iwinski <shawn@iwin.ski> - 6.6.0-1
- Initial package obsoleting php-drush-drush
